# -*- coding: utf-8 -*-
# @Author  : zhousf
# @Function:

import os


class DockerBuild:

    def __init__(self, docker_hub_domain: str = None, docker_hub_namespace: str = None):
        self.docker_hub_domain = docker_hub_domain
        self.docker_hub_namespace = docker_hub_namespace
        self.docker_hub_url = f"{docker_hub_domain}/{docker_hub_namespace}" if docker_hub_domain and docker_hub_namespace else None
        if self.docker_hub_url:
            print("正在登录私有docker仓库")
            os.system(f"docker login {self.docker_hub_domain}")
        pass

    @staticmethod
    def __command(command: str, command_tip: str):
        print(f"正在{command_tip}")
        ret = os.system(command)
        print(ret)
        assert ret == 0, f"{command_tip}失败: {ret}"
        print(f"{command_tip}成功")

    def build_image(self, dockerfile: str, name: str, version: str, command_tip: str, push=False):
        image_name = f"{name}:{version}"
        self.__command(command=f"docker build --network=host -f {dockerfile} -t {image_name} .", command_tip=command_tip)
        if self.docker_hub_url and push:
            self.push_image(image_name)
        return self

    def pull_image(self, tag: str, command_tip: str):
        self.__command(command=f"docker pull {self.docker_hub_url}/{tag}", command_tip=command_tip)
        return self

    def push_image(self, image_name: str):
        if self.docker_hub_url:
            self.__command(command=f"docker tag {image_name} {self.docker_hub_url}/{image_name}", command_tip="镜像打标签")
            self.__command(command=f"docker push {self.docker_hub_url}/{image_name}", command_tip="推送镜像")
        return self


if __name__ == "__main__":
    _docker_hub_domain = "packages.xxx.com"
    _docker_hub_namespace = "docker-xxx"
    build = (DockerBuild(docker_hub_domain=_docker_hub_domain, docker_hub_namespace=_docker_hub_namespace)
             # .pull_image(tag="nvidia/cuda:11.3.1-cudnn8-devel-ubuntu20.04", command_tip="拉取基础镜像")
             # .pull_image(tag="registry.baidubce.com/paddlepaddle/paddle:2.3.1-gpu-cuda11.2-cudnn8 ", command_tip="拉取基础镜像")
             # .pull_image(tag="nvidia/cuda:11.6.1-cudnn8-runtime-ubuntu20.04", command_tip="拉取基础镜像")
             # .build_image(dockerfile="Dockerfile_env_cpu", name="zhousf_fd_py39_cpu", version="1.0.7", push=False, command_tip="构建环境镜像")
             .build_image(dockerfile="Dockerfile_env_gpu", name="zhousf_fd_py39_cuda11.6.1-cudnn8", version="1.0.7", push=False, command_tip="构建环境镜像")
             # .build_image(dockerfile="Dockerfile_pro", name="zhousf_image_classify_gpu", version="0.1", push=True, command_tip="构建工程镜像")
             )



# -*- coding: utf-8 -*-
# @Author  : zhousf
# @Function:
# pip install scikit-learn
from sklearn.metrics import confusion_matrix as confusion_matrix_compute
from sklearn.metrics import fbeta_score, classification_report


class FBetaScore(object):

    def __init__(self, y_true: list, y_pre: list, f_beta: list, fbeta_score_average=None):
        """

        :param y_true:
        :param y_pre:
        :param f_beta: [1, 1.5, 2]
        :param fbeta_score_average:  {'micro', 'macro', 'samples', 'weighted', 'binary'} or None, default='binary'
                This parameter is required for multiclass/multilabel targets.
                If ``None``, the scores for each class are returned. Otherwise, this
                determines the type of averaging performed on the data:

                ``'binary'``:
                    Only report results for the class specified by ``pos_label``.
                    This is applicable only if targets (``y_{true,pred}``) are binary.
                ``'micro'``:
                    Calculate metrics globally by counting the total true positives,
                    false negatives and false positives.
                ``'macro'``:
                    Calculate metrics for each label, and find their unweighted
                    mean.  This does not take label imbalance into account.
                ``'weighted'``:
                    Calculate metrics for each label, and find their average weighted
                    by support (the number of true instances for each label). This
                    alters 'macro' to account for label imbalance; it can result in an
                    F-score that is not between precision and recall.
                ``'samples'``:
                    Calculate metrics for each instance, and find their average (only
                    meaningful for multilabel classification where this differs from
                    :func:`accuracy_score`).
        """
        self.y_true = y_true
        self.y_pre = y_pre
        self.f_beta = f_beta if f_beta is not None else [1]
        self.fbeta_score_average = fbeta_score_average
        self.tn, self.fp, self.fn, self.tp = self.confusion_matrix(y_true=self.y_true, y_pre=self.y_pre)
        self.recall = self.tp / (self.tp + self.fn) if (self.tp + self.fn) > 0 else 0
        self.precision = self.tp / (self.tp + self.fp) if  (self.tp + self.fp) > 0 else 0
        self.f_beta_score = self.f_beta_score_compute(f_beta=self.f_beta, y_true=self.y_true, y_pre=self.y_pre)
        self.report = classification_report(self.y_true, self.y_pre, digits=4)

    def print(self, is_print=True):
        from prettytable import PrettyTable
        table = PrettyTable(field_names=["confusion matrix", "recall", "precision", "f_beta_score"],
                            title="F_beta Score Statistical Table (total={0})".format(len(self.y_true)))
        row = ["tp={0}  fp={1}\nfn={2}  tn={3}".format(self.tp, self.fp, self.fn, self.tn),
               "tp/(tp+fn)={0}/{1}\nrecall={2}".format(self.tp, self.tp + self.fn, self.recall),
               "tp/(tp+fp)={0}/{1}\nprecision={2}".format(self.tp, self.tp + self.fp, self.precision)]
        union = []
        for i, beta in enumerate(self.f_beta):
            union.append("f_{0}={1}".format(beta, self.f_beta_score[i]))
        row.append("\n".join(union))
        table.add_row(row)
        table.align = "l"
        # table.align["confusion matrix"] = "c"
        if is_print:
            print(table)
            print(self.report)
        return table, self.report

    def confusion_matrix(self, y_true: list, y_pre: list):
        """
        1为正例，0为负例
        -------------
        TP 真正例，实际为1，预测为1，正样本预测正确
        FN 假负例，实际为1，预测为0，正样本预测错误
        -------------
        FP 假正例，实际为0，预测为1，负样本预测错误
        TN 真负例，实际为0，预测为0，负样本预测正确

        +------------------+------------------------------------ +
        |                  |              True Class             |
        | Predicted Class  |------------------+------------------+
        |                  |     Positive     |     Negative     |
        +------------------+------------------+------------------+
        |        Y         |        TP        |        FP        |
        +------------------+------------------+------------------+
        |        N         |        FN        |        TN        |
        +------------------+------------------+------------------+
        查全率： Recall = TP/(TP+FN)；     实际为正例的样本中，模型正确预测为正例的比例
        查准率： Precision = TP/(TP+FP)；  模型预测为正例的样本中，实际为正例的比例
        """
        array = confusion_matrix_compute(y_true=y_true, y_pred=y_pre)
        self.tp = array[1][1] if array.shape == (2, 2) else 0
        self.tn = array[0][0]
        self.fp = array[0][1] if array.shape == (2, 2) else 0
        self.fn = array[1][0] if array.shape == (2, 2) else 0
        return self.tn, self.fp, self.fn, self.tp

    def f_beta_score_compute(self, f_beta: list, y_true: list = None, y_pre: list = None):
        """
        Fβ分数 = (1 + β^2) * (精确率 * 召回率) / (β^2 * 精确率 + 召回率)
        β参数决定了精确率和召回率的相对权重
        当 β = 0 时，则Fβ分数等于precision
        当 β < 1 时，则模型偏向精确率
        当 β = 1 时，则Fβ=F1，精确率权重等于召回率权重
        当 β > 1 时，则模型偏向召回率
        :param f_beta:
        :param y_true:
        :param y_pre:
        """
        if y_true is None:
            y_true = self.y_true
        if y_pre is None:
            y_pre = self.y_pre
        f_score = []
        for beta in f_beta:
            f_score.append(fbeta_score(y_true=y_true, y_pred=y_pre, beta=beta, average=self.fbeta_score_average))
        # f_score = (1 + beta ** 2) * (precision * recall) / (beta ** 2 * precision + recall)
        return f_score


if __name__ == "__main__":
    actual_labels =    [0, 1, 0, 1, 1, 0, 0, 0, 0, 0]
    predicted_labels = [0, 1, 0, 1, 0, 1, 1, 0, 0, 0]
    actual_labels =    [0, 0, 0, 0, 0, 0, 1]
    predicted_labels = [0, 0, 0, 0, 1, 1, 1]
    # actual_labels =    [0, 1, 0, 2, 1, 0, 0, 0, 0, 0]
    # predicted_labels = [0, 1, 0, 2, 0, 1, 1, 0, 0, 0]
    score = FBetaScore(y_true=actual_labels, y_pre=predicted_labels, f_beta=[1, 1.1, 2], fbeta_score_average=None)
    score.print()





# -*- coding: utf-8 -*-
# @Author  : zhousf
# @Function:
import sys
import logging
from pathlib import Path
from types import FrameType
from typing import cast
"""

from zhousflib.web.fastapi.log import Logger
from configure import LogDir

Loggers = Logger(LogDir.joinpath("service").joinpath("log.txt"))
logger = Loggers.logger

"""


class Logger:

    def __init__(self, log_file: Path, rotation="5000KB", retention="7 days"):
        log_dir = log_file.parent
        if not log_dir.exists():
            log_dir.mkdir(parents=True, exist_ok=True)
        from loguru import logger as log
        self.logger = log
        # 清空所有设置
        self.logger.remove()
        self.logger.add(sys.stdout,
                        format="<green>{time:YYYYMMDD HH:mm:ss}</green> | "  # 颜色>时间
                               "{process.name} | "  # 进程名
                               "{thread.name} | "  # 进程名
                               "<cyan>{module}</cyan>.<cyan>{function}</cyan>"  # 模块名.方法名
                               ":<cyan>{line}</cyan> | "  # 行号
                               "<level>{level}</level>: "  # 等级
                               "<level>{message}</level>",  # 日志内容
                        )
        self.logger.add(
            sink=log_file,
            format="<green>{time:YYYYMMDD HH:mm:ss}</green> | "  # 颜色>时间
                   "{process.name} | "  # 进程名
                   "{thread.name} | "  # 进程名
                   "<cyan>{module}</cyan>.<cyan>{function}</cyan>"  # 模块名.方法名
                   ":<cyan>{line}</cyan> | "  # 行号
                   "<level>{level}</level>: "  # 等级
                   "<level>{message}</level>",
            serialize=False,
            encoding="utf-8",
            enqueue=True,  # 异步写入
            rotation=rotation,  # 切割
            retention=retention,  # 设置历史保留时长
            backtrace=True,  # 回溯
            diagnose=False,  # 诊断
            compression="zip"  # 文件压缩
        )

    def init_config(self):
        logger_names = ("uvicorn.asgi", "uvicorn.access", "uvicorn.error", "uvicorn",
                        "gunicorn", "gunicorn.access", "gunicorn.error")
        logging.getLogger().handlers = [InterceptHandler(self.logger)]
        for logger_name in logger_names:
            logging_logger = logging.getLogger(logger_name)
            logging_logger.propagate = False
            logging_logger.handlers = [InterceptHandler(self.logger)]

    def get_logger(self):
        return self.logger


class InterceptHandler(logging.Handler):

    def __init__(self, logger):
        super().__init__()
        self.logger = logger

    def emit(self, record: logging.LogRecord) -> None:  # pragma: no cover
        # Get corresponding Loguru level if it exists
        try:
            level = self.logger.level(record.levelname).name
        except ValueError:
            level = str(record.levelno)

        # Find caller from where originated the logged message
        frame, depth = logging.currentframe(), 2
        while frame.f_code.co_filename == logging.__file__:  # noqa: WPS609
            frame = cast(FrameType, frame.f_back)
            depth += 1

        self.logger.opt(depth=depth, exception=record.exc_info).log(
            level, record.getMessage(),
        )




# -*- coding: utf-8 -*-
# @Author  : zhousf
# @Date    : 2023/12/18 
# @Function:
from pathlib import Path


class FastInfer:

    def __init__(self, model_dir: Path = None, device_id=-1):
        """
        :param model_dir:
        :param device_id: cpu上运行：-1 | gpu上运行：0 or 1 or 2...
        """
        self.model_dir = model_dir
        self.device_id = device_id
        self.model = None
        self.backend = None

    def use_http_backend(self, url: str, model_name: str, model_version: str, http_inputs: list, http_outputs: list, **kwargs):
        """
        设置推理后端：http
        :param url:
        :param model_name:
        :param model_version:
        :param http_inputs:
        :param http_outputs:
        :param kwargs:
        :return:
        example:
        .use_http_backend(url="127.0.0.1:5005",
                          model_name="cosnet_onnx",
                          model_version="1",
                          http_inputs=[("input_ids", "INT64"), ("token_type_ids", "INT64"), ("attention_mask", "INT64")],
                          http_outputs=["output"])
        """
        from zhousflib.infer_framework.fast_infer.backend_http import BackendHttp
        self.backend = BackendHttp(model_dir=self.model_dir, device_id=self.device_id)
        self.model = self.backend.model
        return self.backend.build(url=url, model_name=model_name, model_version=model_version, http_inputs=http_inputs, http_outputs=http_outputs, **kwargs)

    def use_original_backend(self, module=None, **kwargs):
        """
        设置推理后端：原始模型-动态图
        :param module:
        :param kwargs:
        :return:
        example:
        .use_original_backend(module=torch.nn.Module())
        """
        from zhousflib.infer_framework.fast_infer.backend_original import BackendOriginal
        self.backend = BackendOriginal(model_dir=self.model_dir, device_id=self.device_id)
        self.model = self.backend.model
        return self.backend.build(module=module, **kwargs)

    def use_onnx_backend(self, from_platform: str = None, module=None, dynamic_axes: dict = None, opset_version=10, example_inputs=None, **kwargs):
        """
        设置推理后端：onnxruntime
        :param from_platform:
        :param module:
        :param dynamic_axes:
        :param opset_version:
        :param example_inputs:
        :param kwargs:
        :return:
        example:
        .use_onnx_backend(from_platform="torch",
                          module=torch.nn.Module(),
                          example_inputs=example_inputs,
                          dynamic_axes={'input_ids': {0: 'batch_size'},
                                        'token_type_ids': {0: 'batch_size'},
                                        'attention_mask': {0: 'batch_size'},
                                        'output': {0: 'batch_size'}})
        """
        from zhousflib.infer_framework.fast_infer.backend_onnx import BackendONNX
        self.backend = BackendONNX(model_dir=self.model_dir, device_id=self.device_id)
        self.model = self.backend.model
        return self.backend.build(from_platform=from_platform, module=module, dynamic_axes=dynamic_axes, opset_version=opset_version, example_inputs=example_inputs, **kwargs)

    def use_tensorrt_backend(self, from_platform: str = None, module=None, dynamic_axes: dict = None, opset_version=10,
                             example_inputs=None, shape: dict = None, **kwargs):
        """
        设置推理后端：tensorrt
        :param from_platform:
        :param module:
        :param dynamic_axes:
        :param opset_version:
        :param example_inputs:
        :param shape:
        :param kwargs:
        :return:
        example:
        若onnx文件存在时则只需传参数shape
        .use_tensorrt_backend(shape={"input_ids": [(10, 128), (10, 128), (10, 128)],
                                     "token_type_ids": [(10, 128), (10, 128), (10, 128)],
                                     "attention_mask": [(10, 128), (10, 128), (10, 128)]})
        若onnx文件不存在时
        .use_tensorrt_backend(from_platform="torch",
                              module=torch.nn.Module(),
                              example_inputs=example_inputs,
                              dynamic_axes={'input_ids': {0: 'batch_size'},
                                          'token_type_ids': {0: 'batch_size'},
                                          'attention_mask': {0: 'batch_size'},
                                          'output': {0: 'batch_size'}},
                              shape={"input_ids": [(10, 128), (10, 128), (10, 128)],
                                     "token_type_ids": [(10, 128), (10, 128), (10, 128)],
                                     "attention_mask": [(10, 128), (10, 128), (10, 128)]})
        """
        from zhousflib.infer_framework.fast_infer.backend_tensorrt import BackendTensorRT
        self.backend = BackendTensorRT(model_dir=self.model_dir, device_id=self.device_id)
        self.model = self.backend.model
        return self.backend.build(from_platform=from_platform, module=module, dynamic_axes=dynamic_axes,
                                  opset_version=opset_version, example_inputs=example_inputs, shape=shape, **kwargs)

    def use_torch_script_backend(self, module=None, example_inputs=None, **kwargs):
        """
        设置推理后端：torch_script
        :param module:
        :param example_inputs:
        :param kwargs:
        :return:
        example:
        .use_torch_script_backend(module=torch.nn.Module(), example_inputs=example_inputs)
        """
        from zhousflib.infer_framework.fast_infer.backend_torch_script import BackendTorchScript
        self.backend = BackendTorchScript(model_dir=self.model_dir, device_id=self.device_id)
        self.model = self.backend.model
        return self.backend.build(module=module, example_inputs=example_inputs, **kwargs)

    def use_fastdeploy_backend(self, **kwargs):
        """
        设置推理后端：fastdeploy
        :param kwargs:
        :return:
        example:
        """
        from zhousflib.infer_framework.fast_infer.backend_fastdeploy import BackendFastDeploy
        self.backend = BackendFastDeploy(model_dir=self.model_dir, device_id=self.device_id)
        self.model = self.backend.model
        return self.backend.build(**kwargs)

    def infer(self, input_data, **kwargs):
        assert self.backend is not None, "请设置backend，例如.use_onnx_backend()"
        return self.backend.inference(input_data=input_data, **kwargs)


def demo_bert():
    import torch
    from zhousflib.infer_framework.ann.torch import torch_to_onnx
    fast_infer = FastInfer(model_dir=Path(r"F:\torch\test"), device_id=0)
    fast_infer.use_tensorrt_backend(from_platform="torch",
                                    module=torch.nn.Module(),
                                    example_inputs=(torch_to_onnx.example_inputs_demo(device_id=0),),
                                    dynamic_axes={'input_ids': {0: 'batch_size'},
                                                  'token_type_ids': {0: 'batch_size'},
                                                  'attention_mask': {0: 'batch_size'},
                                                  'output': {0: 'batch_size'}},
                                    shape={"input_ids": [(10, 128), (10, 128), (10, 128)],
                                           "token_type_ids": [(10, 128), (10, 128), (10, 128)],
                                           "attention_mask": [(10, 128), (10, 128), (10, 128)]})
    return fast_infer.infer(torch_to_onnx.example_inputs_demo()).tolist()


def demo_classification():
    # classification
    import fastdeploy
    model_dir = Path(r"D:\workspace\ZhousfLib\model\PPLCNet_x1_0_infer-v9")
    fast = FastInfer(model_dir=model_dir, device_id=0)
    runtime_option = fastdeploy.RuntimeOption()
    runtime_option.use_ort_backend()
    fast.use_fastdeploy_backend(plugin="fd.vision.classification.PaddleClasModel", runtime_option=runtime_option)
    result = fast.infer(input_data=model_dir.joinpath("test.png"), topk=5)
    print(result)

    # extract image feature
    # model_dir = Path(r"D:\workspace\ZhousfLib\model\general_PPLCNetV2_base_pretrained_v1.0")
    # fast = FastInfer(model_dir=model_dir, device_id=-1)
    # fast.use_fastdeploy_backend(plugin="fd.vision.classification.PPShiTuV2Recognizer", clone_model=True)
    # result = fast.infer(input_data=model_dir.joinpath("test.jpg"))
    # print(result.feature)


def demo_detection():
    import fastdeploy
    model_dir = Path(r"D:\workspace\ZhousfLib\model\global_ppyoloe_plus_crn_l_80e_coco-v10")
    fast = FastInfer(model_dir=model_dir, device_id=0)
    runtime_option = fastdeploy.RuntimeOption()
    runtime_option.use_trt_backend()
    runtime_option.trt_option.serialize_file = str(model_dir.joinpath("model.trt"))
    fast.use_fastdeploy_backend(plugin="fd.vision.detection.PPYOLOE", runtime_option=runtime_option)
    result = fast.infer(input_data=model_dir.joinpath("test.jpg"))
    print(result)


def demo_segmentation():
    model_dir = Path(r"D:\workspace\ZhousfLib\model\local_pp_liteseg_512x512_1k-v5")
    fast = FastInfer(model_dir=model_dir, device_id=-1)
    fast.use_fastdeploy_backend(plugin="fd.vision.segmentation.PaddleSegModel")
    image_file = model_dir.joinpath("test.jpg")
    vis_image_file = image_file.with_name("{0}_vis{1}".format(image_file.stem, image_file.suffix))
    result = fast.infer(input_data=image_file,
                        vis_image_file=vis_image_file,
                        vis_show=True)
    print(result.contain_score_map)


def demo_ocr():
    # detector
    # model_dir = Path(r"D:\workspace\ZhousfLib\model\ch_PP-OCRv4_det_infer")
    # fast = FastInfer(model_dir=model_dir, device_id=0)
    # fast.use_fastdeploy_backend(plugin="fd.vision.ocr.DBDetector")
    # image_file = model_dir.joinpath("test.jpg")
    # vis_image_file = image_file.with_name("{0}_vis{1}".format(image_file.stem, image_file.suffix))
    # res = fast.infer(input_data=image_file,
    #                  vis_image_file=vis_image_file,
    #                  vis_show=True)
    # print(res.boxes)

    # recognizer
    # model_dir = Path(r"D:\workspace\ZhousfLib\model\ch_PP-OCRv4_rec_infer")
    # fast = FastInfer(model_dir=model_dir, device_id=0)
    # fast.use_fastdeploy_backend(plugin="fd.vision.ocr.Recognizer")
    # image_file = model_dir.joinpath("test.jpg")
    # res = fast.infer(input_data=image_file,
    #                  vis_image_file=None,
    #                  vis_show=False)
    # print(res)

    import fastdeploy as fd
    runtime_option = fd.RuntimeOption()
    runtime_option.set_cpu_thread_num(10)
    # runtime_option.use_ort_backend()
    # runtime_option.use_paddle_backend()
    runtime_option.enable_paddle_log_info()  # print log
    # ocr
    fast_det = FastInfer(model_dir=Path(r"D:\workspace\ZhousfLib\model\ch_PP-OCRv4_det_infer"), device_id=0)
    fast_det.use_fastdeploy_backend(plugin="fd.vision.ocr.DBDetector", runtime_option=runtime_option, clone_moel=False)
    fast_det.backend.model.preprocessor.max_side_len = 960
    fast_det.backend.model.preprocessor.static_shape_infer = False
    fast_det.backend.model.postprocessor.det_db_score_mode = 'slow'
    fast_det.backend.model.postprocessor.det_db_unclip_ratio = 1.5
    fast_det.backend.model.postprocessor.use_dilation = True
    fast_cls = FastInfer(model_dir=Path(r"D:\workspace\ZhousfLib\model\ch_PP-OCRv4_cls_infer"), device_id=0)
    fast_cls.use_fastdeploy_backend(plugin="fd.vision.ocr.Classifier", runtime_option=runtime_option, clone_moel=False)
    fast_cls.backend.model.postprocessor.cls_thresh = 0.96
    fast_rec = FastInfer(model_dir=Path(r"D:\workspace\ZhousfLib\model\ch_PP-OCRv4_rec_infer"), device_id=0)
    fast_rec.use_fastdeploy_backend(plugin="fd.vision.ocr.Recognizer", runtime_option=runtime_option, clone_moel=False)
    fast_ocr = FastInfer(model_dir=None, device_id=0)
    fast_ocr.use_fastdeploy_backend(plugin="fd.vision.ocr.PPOCRv4",
                                    det_model=fast_det.backend.model,
                                    cls_model=fast_cls.backend.model,
                                    rec_model=fast_rec.backend.model)
    fast_ocr.backend.model.rec_batch_size = 20
    fast_ocr.backend.model.cls_batch_size = 20

    image_file = Path(r"D:\workspace\ZhousfLib\model\ch_PP-OCRv4_det_infer\test.jpg")
    image_file = Path(r"E:\数据2024\OCR难例\旋转图片\B180度.jpg")
    vis_image_file = image_file.with_name("{0}_ocr_vis{1}".format(image_file.stem, image_file.suffix))
    res = fast_ocr.infer(input_data=image_file,
                         vis_image_file=vis_image_file,
                         vis_show=True)
    print(res)


if __name__ == "__main__":
    demo_ocr()
    # demo_classification()
    # demo_detection()
    # demo_segmentation()
    pass


# -*- coding: utf-8 -*-
# @Author  : zhousf
# @Function:
from starlette.exceptions import HTTPException
from fastapi.responses import JSONResponse
from fastapi.exceptions import RequestValidationError


async def global_exception_handler(request, exc):
    if exc.status_code == 500:
        err_msg = 'Server Internal Error'
    else:
        err_msg = exc.detail
    return JSONResponse({
        'resultCode': exc.status_code,
        'errorString': err_msg,
        'status': 'Failed'
    })


async def validate_exception_handler(request, exc):
    return JSONResponse({
        'resultCode': 400,
        'errorString': exc.errors(),
        'status': 'Failed'
    })


EXCEPTION_HANDLERS = {
    HTTPException: global_exception_handler,
    RequestValidationError: validate_exception_handler
}



# -*- coding: utf-8 -*-
# @Author  : zhousf
# @Function:
from pathlib import Path

import requests
from zhousflib.util import list_util
from zhousflib.thread.thread_util import MultiThread


# noinspection PyBroadException
class DownloadRequests:

    def __init__(self):
        self.task: list[[str, Path, int]] = []

    @staticmethod
    def get_file(url: str, save_file: Path, timeout: int = 10):
        try:
            response = requests.get(url, timeout=timeout)
            if response.status_code != 200:
                return False, "{0} {1}".format(response.reason, url)
            else:
                with save_file.open("wb") as file:
                    file.write(response.content)
                return True, url
        except Exception as e:
            return False, "{0} {1}".format(e, url)

    def add(self, url: str, save_file: Path, timeout: int = 10):
        self.task.append((url, save_file, timeout))

    @staticmethod
    def __get_files(thread_name, func_arg):
        file_list = func_arg
        result = []
        if len(file_list) > 0:
            for i, item in enumerate(file_list):
                url, file, timeout = item
                try:
                    response = requests.get(url, timeout=timeout)
                    if response.status_code != 200:
                        result.append((False, "{0} {1} {2}".format(file.name, response.reason, response.status_code)))
                        continue
                    with open(str(file), "wb") as file:
                        file.write(response.content)
                    result.append((True, file))
                except Exception as e:
                    result.append((False, "{0} {1}".format(file.name, e)))
        return result

    def get_file_threads(self, threads_num=8):
        threads = []
        task_list = list_util.split_list(self.task, split_size=threads_num)
        for i, item in enumerate(task_list):
            thread = MultiThread(name="thread_{0}".format(i), func=self.__get_files, func_arg=item, thread_lock=None)
            thread.start()
            threads.append(thread)
        for t in threads:
            t.join()
        self.task.clear()
        result = []
        for thread in threads:
            result.extend(thread.get_result())
        return result




# Copyright (c) 2020 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from zhousflib.font import dynamic_font
import argparse
import os
import sys
import cv2
import numpy as np
import json

import random
from PIL import Image, ImageDraw, ImageFont
import math
# from paddle import inference
# pip install paddlepaddle-gpu==2.0.0rc1 -i https://mirror.baidu.com/pypi/simple


def parse_args():
    def str2bool(v):
        return v.lower() in ("true", "t", "1")

    parser = argparse.ArgumentParser()
    # params for prediction engine
    parser.add_argument("--use_gpu", type=str2bool, default=True)
    parser.add_argument("--ir_optim", type=str2bool, default=True)
    parser.add_argument("--use_tensorrt", type=str2bool, default=False)
    parser.add_argument("--use_fp16", type=str2bool, default=False)
    parser.add_argument("--gpu_mem", type=int, default=500)

    # params for text detector
    parser.add_argument("--image_dir", type=str)
    parser.add_argument("--det_algorithm", type=str, default='DB')
    parser.add_argument("--det_model_dir", type=str)
    parser.add_argument("--det_limit_side_len", type=float, default=960)
    parser.add_argument("--det_limit_type", type=str, default='max')

    # DB parmas
    parser.add_argument("--det_db_thresh", type=float, default=0.3)
    parser.add_argument("--det_db_box_thresh", type=float, default=0.5)
    parser.add_argument("--det_db_unclip_ratio", type=float, default=1.6)
    parser.add_argument("--max_batch_size", type=int, default=10)
    # EAST parmas
    parser.add_argument("--det_east_score_thresh", type=float, default=0.8)
    parser.add_argument("--det_east_cover_thresh", type=float, default=0.1)
    parser.add_argument("--det_east_nms_thresh", type=float, default=0.2)

    # SAST parmas
    parser.add_argument("--det_sast_score_thresh", type=float, default=0.5)
    parser.add_argument("--det_sast_nms_thresh", type=float, default=0.2)
    parser.add_argument("--det_sast_polygon", type=bool, default=False)

    # params for text recognizer
    parser.add_argument("--rec_algorithm", type=str, default='CRNN')
    parser.add_argument("--rec_model_dir", type=str)
    parser.add_argument("--rec_image_shape", type=str, default="3, 32, 320")
    parser.add_argument("--rec_char_type", type=str, default='ch')
    parser.add_argument("--rec_batch_num", type=int, default=6)
    parser.add_argument("--max_text_length", type=int, default=25)
    parser.add_argument(
        "--rec_char_dict_path",
        type=str,
        default="./ppocr/utils/ppocr_keys_v1.txt")
    parser.add_argument("--use_space_char", type=str2bool, default=True)
    parser.add_argument(
        "--vis_font_path", type=str, default="./doc/simfang.ttf")
    parser.add_argument("--drop_score", type=float, default=0.5)

    # params for text classifier
    parser.add_argument("--use_angle_cls", type=str2bool, default=False)
    parser.add_argument("--cls_model_dir", type=str)
    parser.add_argument("--cls_image_shape", type=str, default="3, 48, 192")
    parser.add_argument("--label_list", type=list, default=['0', '180'])
    parser.add_argument("--cls_batch_num", type=int, default=6)
    parser.add_argument("--cls_thresh", type=float, default=0.9)

    parser.add_argument("--enable_mkldnn", type=str2bool, default=False)
    parser.add_argument("--use_pdserving", type=str2bool, default=False)

    return parser.parse_args()


def log(logger, msg):
    if logger is not None:
        logger.info(msg)

#
# def create_predictor(args, mode, logger):
#     if mode == "det":
#         model_dir = args.det_model_dir
#     elif mode == 'cls':
#         model_dir = args.cls_model_dir
#     else:
#         model_dir = args.rec_model_dir
#
#     if model_dir is None:
#         log(logger=logger, msg="not find {} model file path {}".format(mode, model_dir))
#         sys.exit(0)
#     model_file_path = model_dir + "/inference.pdmodel"
#     params_file_path = model_dir + "/inference.pdiparams"
#     if not os.path.exists(model_file_path):
#         log(logger=logger, msg="not find model file path {}".format(model_file_path))
#         sys.exit(0)
#     if not os.path.exists(params_file_path):
#         log(logger=logger, msg="not find params file path {}".format(params_file_path))
#         sys.exit(0)
#
#     config = inference.Config(model_file_path, params_file_path)
#
#     if args.use_gpu:
#         config.enable_use_gpu(args.gpu_mem, int(args.gpu_assigned))
#         if args.use_tensorrt:
#             config.enable_tensorrt_engine(
#                 precision_mode=inference.PrecisionType.Half
#                 if args.use_fp16 else inference.PrecisionType.Float32,
#                 max_batch_size=args.max_batch_size)
#     else:
#         config.disable_gpu()
#         config.set_cpu_math_library_num_threads(6)
#         if args.enable_mkldnn:
#             # cache 10 different shapes for mkldnn to avoid memory leak
#             config.set_mkldnn_cache_capacity(10)
#             config.enable_mkldnn()
#             args.rec_batch_num = 1
#
#     # config.enable_memory_optim()
#     config.disable_glog_info()
#
#     config.delete_pass("conv_transpose_eltwiseadd_bn_fuse_pass")
#     config.switch_use_feed_fetch_ops(False)
#
#     # create predictor
#     predictor = inference.create_predictor(config)
#     input_names = predictor.get_input_names()
#     for name in input_names:
#         input_tensor = predictor.get_input_handle(name)
#     output_names = predictor.get_output_names()
#     output_tensors = []
#     for output_name in output_names:
#         output_tensor = predictor.get_output_handle(output_name)
#         output_tensors.append(output_tensor)
#     return predictor, input_tensor, output_tensors
#

def draw_text_det_res(dt_boxes, img_path):
    src_im = cv2.imread(img_path)
    for box in dt_boxes:
        box = np.array(box).astype(np.int32).reshape(-1, 2)
        cv2.polylines(src_im, [box], True, color=(255, 255, 0), thickness=2)
    return src_im


def resize_img(img, input_size=600):
    """
    resize img and limit the longest side of the image to input_size
    """
    img = np.array(img)
    im_shape = img.shape
    im_size_max = np.max(im_shape[0:2])
    im_scale = float(input_size) / float(im_size_max)
    img = cv2.resize(img, None, None, fx=im_scale, fy=im_scale)
    return img


def draw_ocr(image,
             boxes,
             txts=None,
             scores=None,
             drop_score=0.5,
             font_path="./doc/simfang.ttf"):
    """
    Visualize the results of OCR detection and recognition
    args:
        image(Image|array): RGB image
        boxes(list): boxes with shape(N, 4, 2)
        txts(list): the texts
        scores(list): txxs corresponding scores
        drop_score(float): only scores greater than drop_threshold will be visualized
        font_path: the path of font which is used to draw text
    return(array):
        the visualized img
    """
    if scores is None:
        scores = [1] * len(boxes)
    box_num = len(boxes)
    for i in range(box_num):
        if scores is not None and (scores[i] < drop_score or
                                   math.isnan(scores[i])):
            continue
        box = np.reshape(np.array(boxes[i]), [-1, 1, 2]).astype(np.int64)
        image = cv2.polylines(np.array(image), [box], True, (255, 0, 0), 2)
    if txts is not None:
        img = np.array(resize_img(image, input_size=600))
        txt_img = text_visual(
            txts,
            scores,
            img_h=img.shape[0],
            img_w=600,
            threshold=drop_score,
            font_path=font_path)
        img = np.concatenate([np.array(img), np.array(txt_img)], axis=1)
        return img
    return image


def draw_ocr_box_txt(image,
                     boxes,
                     txts,
                     scores=None,
                     drop_score=0.5,
                     font_path="./doc/simfang.ttf"):
    """
    如果竖排文字重叠请安装：pillow==9.0.1
    """
    if image.mode != "RGB":
        image = image.convert('RGB')
    maxheight = max([i[2][1] for i in boxes])
    h, w = image.height, image.width
    # true : 图片方向有角度
    turn = maxheight > h
    img_left = image.copy()
    img_right = Image.new('RGB', (h, w), (255, 255, 255)) if turn else Image.new('RGB', (w, h), (255, 255, 255))
    random.seed(0)
    draw_left = ImageDraw.Draw(img_left)
    draw_right = ImageDraw.Draw(img_right)
    for idx, (box, txt) in enumerate(zip(boxes, txts)):
        if scores is not None and scores[idx] < drop_score:
            continue
        color = (random.randint(128, 255), random.randint(128, 255),
                 random.randint(128, 255), 128)
        draw_left.polygon(box, fill=color)
        # draw_right.polygon(
        #     [
        #         box[0][0], box[0][1], box[1][0], box[1][1], box[2][0],
        #         box[2][1], box[3][0], box[3][1]
        #     ],
        #     outline=color)
        draw_right.polygon(
            [
                box[0][0], box[0][1], box[1][0], box[1][1], box[2][0],
                box[2][1], box[3][0], box[3][1]
            ],
            fill=color)
    for idx, (box, txt) in enumerate(zip(boxes, txts)):
        if scores is not None and scores[idx] < drop_score:
            continue
        box_height = math.sqrt((box[0][0] - box[3][0])**2 + (box[0][1] - box[3][
            1])**2)
        box_width = math.sqrt((box[0][0] - box[1][0])**2 + (box[0][1] - box[1][
            1])**2)
        if box_height > 2 * box_width:
            font_size = max(int(box_width * 0.8), 8)
            font = ImageFont.truetype(font_path, font_size, encoding="utf-8")
            cur_y = box[0][1]
            for c in txt:
                if hasattr(font, "getsize"):
                    char_size = font.getsize(c)
                else:
                    char_size = font.getbbox(c)
                draw_right.text(
                    (box[0][0] + 3, cur_y), c, fill=(0, 0, 0), font=dynamic_font(text=c, font_size=font_size))
                cur_y += char_size[1]
        else:
            font_size = max(int(box_height * 0.7), 8)
            draw_right.text(
                (box[0][0], box[0][1]), txt, fill=(0, 0, 0), font=dynamic_font(text=txt, font_size=font_size))

    img_left = Image.blend(image, img_left, 0.5)
    # img_show = Image.new('RGB', (w * 2, h + w), (255, 255, 255))
    img_show = Image.new('RGB', (w * 2, h + w), (255, 255, 255)) if turn else Image.new('RGB', (w * 2, h), (255, 255, 255))
    img_show.paste(img_left, (0, 0, w, h))
    if turn:
        img_show.paste(img_right, (w, 0, w + h, w))
    else:
        img_show.paste(img_right, (w, 0, w * 2, h))
    return np.array(img_show)


def str_count(s):
    """
    Count the number of Chinese characters,
    a single English character and a single number
    equal to half the length of Chinese characters.
    args:
        s(string): the input of string
    return(int):
        the number of Chinese characters
    """
    import string
    count_zh = count_pu = 0
    s_len = len(s)
    en_dg_count = 0
    for c in s:
        if c in string.ascii_letters or c.isdigit() or c.isspace():
            en_dg_count += 1
        elif c.isalpha():
            count_zh += 1
        else:
            count_pu += 1
    return s_len - math.ceil(en_dg_count / 2)


def text_visual(texts,
                scores,
                img_h=400,
                img_w=600,
                threshold=0.,
                font_path="./doc/simfang.ttf"):
    """
    create new blank img and draw txt on it
    args:
        texts(list): the text will be draw
        scores(list|None): corresponding score of each txt
        img_h(int): the height of blank img
        img_w(int): the width of blank img
        font_path: the path of font which is used to draw text
    return(array):
    """
    if scores is not None:
        assert len(texts) == len(
            scores), "The number of txts and corresponding scores must match"

    def create_blank_img():
        blank_img = np.ones(shape=[img_h, img_w], dtype=np.int8) * 255
        blank_img[:, img_w - 1:] = 0
        blank_img = Image.fromarray(blank_img).convert("RGB")
        draw_txt = ImageDraw.Draw(blank_img)
        return blank_img, draw_txt

    blank_img, draw_txt = create_blank_img()

    font_size = 20
    txt_color = (0, 0, 0)
    font = ImageFont.truetype(font_path, font_size, encoding="utf-8")

    gap = font_size + 5
    txt_img_list = []
    count, index = 1, 0
    for idx, txt in enumerate(texts):
        index += 1
        if scores[idx] < threshold or math.isnan(scores[idx]):
            index -= 1
            continue
        first_line = True
        while str_count(txt) >= img_w // font_size - 4:
            tmp = txt
            txt = tmp[:img_w // font_size - 4]
            if first_line:
                new_txt = str(index) + ': ' + txt
                first_line = False
            else:
                new_txt = '    ' + txt
            draw_txt.text((0, gap * count), new_txt, txt_color, font=font)
            txt = tmp[img_w // font_size - 4:]
            if count >= img_h // gap - 1:
                txt_img_list.append(np.array(blank_img))
                blank_img, draw_txt = create_blank_img()
                count = 0
            count += 1
        if first_line:
            new_txt = str(index) + ': ' + txt + '   ' + '%.3f' % (scores[idx])
        else:
            new_txt = "  " + txt + "  " + '%.3f' % (scores[idx])
        draw_txt.text((0, gap * count), new_txt, txt_color, font=font)
        # whether add new blank img or not
        if count >= img_h // gap - 1 and idx + 1 < len(texts):
            txt_img_list.append(np.array(blank_img))
            blank_img, draw_txt = create_blank_img()
            count = 0
        count += 1
    txt_img_list.append(np.array(blank_img))
    if len(txt_img_list) == 1:
        blank_img = np.array(txt_img_list[0])
    else:
        blank_img = np.concatenate(txt_img_list, axis=1)
    return np.array(blank_img)


def base64_to_cv2(b64str):
    import base64
    data = base64.b64decode(b64str.encode('utf8'))
    data = np.fromstring(data, np.uint8)
    data = cv2.imdecode(data, cv2.IMREAD_COLOR)
    return data


def draw_boxes(image, boxes, scores=None, drop_score=0.5):
    if scores is None:
        scores = [1] * len(boxes)
    for (box, score) in zip(boxes, scores):
        if score < drop_score:
            continue
        box = np.reshape(np.array(box), [-1, 1, 2]).astype(np.int64)
        image = cv2.polylines(np.array(image), [box], True, (255, 0, 0), 2)
    return image


if __name__ == '__main__':
    test_img = "./doc/test_v2"
    predict_txt = "./doc/predict.txt"
    f = open(predict_txt, 'r')
    data = f.readlines()
    img_path, anno = data[0].strip().split('\t')
    img_name = os.path.basename(img_path)
    img_path = os.path.join(test_img, img_name)
    image = Image.open(img_path)

    data = json.loads(anno)
    boxes, txts, scores = [], [], []
    for dic in data:
        boxes.append(dic['points'])
        txts.append(dic['transcription'])
        scores.append(round(dic['scores'], 3))

    new_img = draw_ocr(image, boxes, txts, scores)

    cv2.imwrite(img_name, new_img)



# -*- coding: utf-8 -*-
# @Author  : zhousf-a
# @Function: pip install pympler
"""
获取对象的内存大小
方式1：采用 import sys; sys.getsizeof()不遍历对象内部的元素指向内存
方式2：采用 from pympler import asizeof; asizeof.asizeof()遍历对象内部元素，推荐使用
"""
from pympler import asizeof


def get_memory_byte_size(data):
    """
    获取对象占用内存的大小，单位：byte
    :param data:
    :return:
    """
    return asizeof.asizeof(data)


def get_memory_mb_size(data):
    """
    获取对象占用内存的大小，单位：MB
    :param data:
    :return:
    """
    size_byte = get_memory_byte_size(data)
    return size_byte / (1024 * 1024)



# -*- coding: utf-8 -*-
# @Author  : zhousf
# @Function: 相似度计算
import jieba
from typing import List
from zhousflib.util import re_util
from zhousflib.metrics.cosine import Cosine
from zhousflib.ml.feature_vector import FeatureVector, TypeFeatureVector

"""
字符串匹配算法：这是最基本的文本相似度计算方法，主要通过将两个文本字符串进行逐个字符的比较，计算相同字符的数量占总字符数的比例来判断文本的相似度。但是，这种方法对于大量文本的比对速度较慢，且只能检测出完全相同的文本
哈希算法：哈希算法可以快速计算出文本的哈希值，然后通过比对哈希值来判断文本的相似度。但是，哈希算法存在哈希冲突的问题，即不同的文本可能会产生相同的哈希值，从而导致误判
N-gram算法：N-gram算法是一种基于文本分词的方法，将文本分成N个连续的词组，然后比对词组的相似度来判断文本的相似度。N-gram算法可以识别出部分相似的文本，相对于字符串匹配算法和哈希算法，其检测精度更高。
向量空间模型算法：向量空间模型算法是一种基于文本向量化的方法，将文本转换成向量，然后计算向量之间的相似度来判断文本的相似度。这种方法可以识别出语义相似的文本，相对于其他算法，其检测精度更高。

MinHash算法：MinHash算法是一种基于哈希的文本查重方法，它通过随机排列文档中的词项并使用哈希函数来比较文档的相似性。
SimHash算法：

运行速度：KSentence > Simhash > Minhash
准确率：KSentence > Minhash > Simhash
召回率：Simhash > Minhash > KSentence
工程应用上，海量文本用Simhash，短文本用Minhash，追求速度用KSentence。
"""

"""
余弦相似度：from sklearn.metrics.pairwise import cosine_similarity   
欧氏距离：  from sklearn.metrics.pairwise import euclidean_distances
曼哈顿距离：from sklearn.metrics.pairwise import manhattan_distances
"""


def text_to_vector(text: List[str], vector_type=TypeFeatureVector.TYPE_COUNT_VECTOR):
    return FeatureVector(vector_type=vector_type).fit_transform(text)


def compute_similarity(text: List[str], vector_type=TypeFeatureVector.TYPE_COUNT_VECTOR, filter_punctuation=True,
                       cut_all=True, filter_result=False, filter_threshold: float = 0):
    """

    :param text:
    :param vector_type:
    :param filter_punctuation:
    :param cut_all:
    :param filter_result:
    :param filter_threshold:
    :return:
    """
    _text = []
    for txt in text:
        if filter_punctuation:
            txt = re_util.get_digit_letter_chinese(str(txt))
        _text.append(str(jieba.lcut(txt, cut_all=cut_all)))
    cosine = Cosine()
    vector = text_to_vector(_text, vector_type)
    similarity_matrix = cosine.cosine_vector_with_matrix(vector, filter_result=filter_result, filter_threshold=filter_threshold)
    return similarity_matrix


if __name__ == "__main__":
    import time
    start = time.time()
    documents = ["This is the first document",
                 "This document is the second document",
                 "This is the third document"]
    results = compute_similarity(text=documents, vector_type=TypeFeatureVector.TYPE_COUNT_VECTOR,
                                 filter_punctuation=False, cut_all=False, filter_result=False, filter_threshold=0.1)
    print(results)
    print("耗时", time.time() - start)




# -*- coding: utf-8 -*-
# @Author  : zhousf-a
# @Function:
from typing import Set


def jaccard_vector(set1: Set, set2: Set):
    """

    :param set1:
    :param set2:
    :return:
    """
    intersection = len(set1.intersection(set2))
    union = len(set1.union(set2))
    return intersection / union


if __name__ == "__main__":
    similarity = jaccard_vector(set("This is the first document".split()),
                                set("The document is the second document".split()))
    print(similarity)
    pass



# -*- coding: utf-8 -*-
# @Author  : zhousf
# @Function:
"""
pip install oss2
"""
import os

import oss2
from loguru import logger
from zhousflib.time import time_cost
from zhousflib.util import list_util
from zhousflib.thread.thread_util import MultiThread


class HandleOSS(object):
    def __init__(self, key_id, key_secret, endpoint, bucket):
        """
        :param key_id:
        :param key_secret:
        :param endpoint: 例如：https://oss-cn-hangzhou.aliyuncs.com
        :param bucket: bucket名字，例如：product-ai
        """
        self.auth = oss2.Auth(key_id, key_secret)
        self.bucket = oss2.Bucket(self.auth, endpoint, bucket)

    @time_cost(logger)
    def download_file(self, oss_file, save_dir):
        file_name = oss_file.split('/')[-1]
        save_path = os.path.join(str(save_dir), file_name)
        if not os.path.exists(save_dir):
            os.mkdir(save_dir)
        result = self.bucket.get_object_to_file(oss_file, str(save_path))
        if result.status == 200:
            logger.info("download success: {0}".format(save_path))
        else:
            logger.info(result.status)

    @time_cost(logger)
    def download_files(self, oss_dir, save_dir):
        if not os.path.exists(save_dir):
            os.mkdir(save_dir)
        obj = oss2.ObjectIterator(self.bucket, prefix=oss_dir)
        total = 0
        for i in obj:
            if i.key.endswith(os.sep):
                continue
            if str(i.key).find(".") == -1:
                continue
            total += 1
            self.download_file(i.key, save_dir)
        logger.info("total: {}".format(total))

    @time_cost(logger)
    def download_files_multithread(self, oss_dir, save_dir, thread=2):
        total = 0
        if not os.path.exists(save_dir):
            os.mkdir(save_dir)
        obj = oss2.ObjectIterator(self.bucket, prefix=oss_dir)
        task = []
        for i in obj:
            if i.key.endswith(os.sep):
                continue
            if str(i.key).find(".") == -1:
                continue
            total += 1
            task.append((i.key, save_dir))
        task_group = list_util.chunk_list(task, chunk_size=thread)
        threads = []
        for i, task in enumerate(task_group):
            thread = MultiThread(name="thread_{0}".format(i), func=self.__download, func_arg=task, thread_lock=None)
            thread.start()
            threads.append(thread)
        for t in threads:
            t.join()
        logger.info("total: {}".format(total))
        return total

    def __download(self, thread_name, func_arg):
        for task in func_arg:
            self.download_file(task[0], task[1])
        return thread_name









# -*- coding: utf-8 -*-
# @Author  : zhousf
# @Date    : 2023/12/21 
# @Function:
import onnxruntime
"""
【onnx && cuda的版本对应关系】
onnx对应cuda的版本：https://onnxruntime.ai/docs/execution-providers/CUDA-ExecutionProvider.html#requirements
注意onnxruntime与opset版本的对应关系


############## 【安装onnxruntime】 ##############
选择版本：https://onnxruntime.ai/docs/execution-providers/CUDA-ExecutionProvider.html#requirements
【cpu】
pip install onnxruntime==1.13.1 -i http://mirrors.aliyun.com/pypi/simple/ --trusted-host mirrors.aliyun.com
【gpu】
pip install onnxruntime-gpu==1.13.1 -i http://mirrors.aliyun.com/pypi/simple/ --trusted-host mirrors.aliyun.com
验证：
import onnxruntime
onnxruntime.get_device()


############## 【验证导出onnx是否正确】 ##############
可视化网络结构：https://netron.app/
当output有if条件则会存在问题，更换opset版本(opset=10)或降低torch版本(1.8.0)
"""


def load_onnx(**kwargs):
    """
    加载onnx模型
    :return:
    ort_session = load_onnx(model_dir=Path(r"model.onnx"))
    ort_input = ort_session.get_inputs()
    args = example_inputs_demo()
    ort_inputs = {ort_input[0].name: to_numpy(args[0]),
                  ort_input[1].name: to_numpy(args[1]),
                  ort_input[2].name: to_numpy(args[2])}
    ort_outs = ort_session.run(None, ort_inputs)
    print(ort_outs[0])
    """
    model_file = kwargs.get("model_file")
    # cpu上运行：-1 | gpu上运行：0 or 1 or 2...
    device_id = kwargs.get("device_id", -1)
    # 预测优化
    ir_optim = kwargs.get("ir_optim", True)
    # 多核处理节点 Sets the number of threads used to parallelize the execution within nodes
    intra_op_num_threads = kwargs.get("intra_op_num_threads", 8)
    # 并行处理子图 Sets the number of threads used to parallelize the execution of the graph (across nodes).
    inter_op_num_threads = kwargs.get("inter_op_num_threads", 100)
    assert model_file is not None, "onnx file not found: {0}".format(model_file)
    config = onnxruntime.SessionOptions()
    config.intra_op_num_threads = intra_op_num_threads
    if ir_optim:
        # 启用图优化
        config.graph_optimization_level = onnxruntime.GraphOptimizationLevel.ORT_ENABLE_ALL
        # 内部子图并行计算
        config.execution_mode = onnxruntime.ExecutionMode.ORT_PARALLEL
        config.inter_op_num_threads = inter_op_num_threads
    if device_id == -1:
        onnx_session = onnxruntime.InferenceSession(str(model_file))
    else:
        onnx_session = onnxruntime.InferenceSession(str(model_file), providers=['CUDAExecutionProvider'],
                                                    provider_options=[{'device_id': device_id}])
    return onnx_session


class OnnxPredictor(object):

    def __init__(self, **kwargs):
        self.predictor = load_onnx(**kwargs)

    @property
    def session(self):
        return self.predictor

    def run(self, output_names, input_feed):
        return self.predictor.run(output_names=output_names, input_feed=input_feed)





# -*- coding: utf-8 -*-
# @Author  : zhousf
# @Function:
"""
pip install huaweicloud-sdk-python-obs
"""
import traceback

from obs import ObsClient


class OBSHuaWei:
    def __init__(self, key_id, key_secret, endpoint, bucket):
        self.bucketName = bucket
        self.obsClient = ObsClient(access_key_id=key_id, secret_access_key=key_secret, server=endpoint)

    def obs2local(self, obs_path, local_path):
        return self.obsClient.getObject(self.bucketName, obs_path, downloadPath=local_path)

    def local2obs(self, obs_path, local_path):
        try:
            resp = self.obsClient.putFile(self.bucketName, obs_path, local_path)
            if resp.status < 300:
                return True, "successful"
            else:
                return False, "{0}: {1}".format(resp.errorCode, resp.errorMessage)
        except Exception as e:
            return False, "{0}".format(e)
        finally:
            self.obsClient.close()

    def local2obs_return_url(self, obs_path, local_path, expires=2592000):
        success, msg = self.local2obs(obs_path=obs_path, local_path=local_path)
        if not success:
            return success, msg
        try:
            resp = self.obsClient.createSignedUrl('GET', self.bucketName, obs_path, expires=expires)
            return True, resp.signedUrl
        except Exception as e:
            return False, "{0}".format(traceback.print_exc())


